/*
 * BongoDB, an embedded document-based engine
 * Copyright (C) 2015 by Javier Sancho Fernandez <jsf at jsancho dot org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "bongodb.h"
#include "jsmn.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int
test_bongodb()
{
  bongo_client_t *bongo_client;
  bongo_database_t *database;
  bongo_collection_t *collection;
  const char *document = "{\"name\": \"Jack\", \"age\": 27, \"list\": [1, 2, 3]}";
  bongo_error_t error;
  
  bongo_client = bongo_client_new ();
  printf ("Databases: %d\n", bongo_client->n_databases);
  database = bongo_client_get_database (bongo_client, "db_test");
  printf ("Databases: %d\n", bongo_client->n_databases);

  printf ("Collections: %d\n", database->database_node->n_collections);
  collection = bongo_client_get_collection (bongo_client, "db_test", "col_test");
  printf ("Collections: %d\n", database->database_node->n_collections);

  if (!bongo_collection_insert (collection, BONGO_INSERT_NONE, document, NULL, &error)) {
    fprintf (stderr, "%s\n", error.message);
  }

  bongo_collection_destroy (collection);
  bongo_database_destroy (database);
  bongo_client_destroy (bongo_client);
  
  printf ("test_bongodb: PASSED\n");
  return 0;
}

int
test_jsmn()
{
  jsmn_parser parser;
  jsmntok_t *tokens;
  int n_tokens;
  const char *js = "{\"name\": \"Jack\", \"age\": 27, \"list\": [1, 2, 3]}";

  jsmn_init (&parser);
  n_tokens = jsmn_parse (&parser, js, strlen (js), NULL, 1);
  tokens = malloc (sizeof (jsmntok_t) * n_tokens);
  jsmn_init (&parser);
  n_tokens = jsmn_parse (&parser, js, strlen (js), tokens, n_tokens);
  printf ("Tokens: %d\n", n_tokens);

  for (int i = 0; i < n_tokens; i++)
    {
      printf ("   %d, %d, %d, %d: %.*s\n", tokens[i].type, tokens[i].start, tokens[i].end, tokens[i].size, tokens[i].end - tokens[i].start, js + tokens[i].start);
    }

  free (tokens);
  
  printf ("test_jsmn: PASSED\n");
  return 0;
}

int
main()
{
  test_bongodb ();
  test_jsmn ();
  return 0;
}
